from __future__ import annotations
import sys
import argparse
from srutil import util

from . import browse, __version__, __package__


def get_argument():
    parser = argparse.ArgumentParser(prog=__package__, usage=util.stringbuilder(__package__, " [options]"))
    parser.add_argument('-v', '--version', action='version', help='show version number and exit', version=__version__)
    group = parser.add_argument_group("to browse web")
    group.add_argument("-q", "--query", dest='query', metavar='', default=None, type=str, help="query to search")
    group.add_argument("-u", "--url", dest="url", default=None, metavar='', type=str, help="url to open")
    group.add_argument("-e", "--engine", dest="engine", metavar='', default="google", required=False,
                       choices=["google", "duckduckgo", "yahoo", "bing", "yandex", "qwant", "wikipedia", "youtube"],
                       help="search engine to search (Optional)")
    parser.add_argument_group(group)
    options = parser.parse_args()
    if not options.query and not options.url:
        parser.error("one of the following arguments are required: -q/--query or -u/--url")
    if options.query and options.url:
        parser.error("any one of the following arguments should be given: -q/--query or -u/--url")
    return options


def main():
    options = get_argument()
    if options.query:
        browse.search(query=options.query, engine=options.engine)
    elif options.url:
        browse.open(url=options.url)


if __name__ == "__main__":
    sys.exit(main())



from __future__ import annotations

import urllib.request
from urllib.parse import urlparse

from srutil import util


class OnlineException(Exception):
    pass


def encoded_query(_query: str) -> str:
    replaces = {'%': '%25', ':': '%3A', ';': '%3B', '@': '%40', '#': '%23', '$': '%24', '=': '%3D', ',': '%2C',
                '?': '%3F', '/': '%2F', '&': '%26', '+': '%2B', ' ': '+'}
    query = ''
    for letters in _query:
        query += replaces.get(letters) if letters in replaces else letters
    return query


def get_baseurl_for(web_engine: str) -> str:
    engine = {"base-url": "duck.co", "duckduckgo": "!?", "wikipedia": "!w", "yandex": "!yri",
              "youtube": "!yt", "google": "!g", "qwant": "!qw", "yahoo": "!y", "bing": "!b"}
    base_url = util.stringbuilder(engine.get("base-url"), "/?q=", engine.get(web_engine))
    return base_url


def is_valid_url(url: str) -> bool:
    if util.isnetworkconnected():
        result = urlparse(url)
        response = urllib.request.urlopen(url)
        return all([result.scheme, result.netloc, result.path]) and response.code < 400
    else:
        raise OnlineException("Invalid URL.")



import os.path
import webbrowser
from srutil import util
from typing import Literal

from . import util as bu


class Browse:

    @staticmethod
    def open(url: str):
        """
        open url in default browser.

        :param url: url to open
        """
        url = Browse.__validate_url(url)
        webbrowser.open(url)

    @staticmethod
    def search(query: str,
               engine: Literal[
                   "google", "duckduckgo", "yahoo", "bing", "yandex", "qwant", "wikipedia", "youtube"] = "google"):
        """
        search for query in default browser.

        :param query: query to search
        :param engine: search engine
        """
        _url = Browse.__get_as_url(bu.encoded_query(query), engine)
        Browse.open(url=_url)

    @staticmethod
    def __get_as_url(query, web_engine) -> str:
        url = util.stringbuilder(bu.get_baseurl_for(web_engine), query, separator="+")
        return url

    @staticmethod
    def __validate_url(url) -> str:
        if not os.path.exists(url):
            if not url.startswith("http"):
                url = "https://" + url
            bu.is_valid_url(url)
        return url
